function obj=deflate(varargin)
% deflate - constructor for deflated matrix
% 
% A deflated matrix is used for storage of large non-sparse N-D matrices of
% up to 2^32 elements.
% 
% CALL
%     m=deflate(M), OR:
%     m=deflate(M,<PROPERTY>,<VALUE>,<PROPERTY>,<VALUE>), OR:
%     m=deflate(<PROPERTY>,<VALUE>,<PROPERTY>,<VALUE>,..)
%
% INPUT
%     M      : numeric or char matrix, deflated matrix, or structure with
%              equivalent fields
% 
%     <PROPERTY>,<VALUE>: property value pair. 
%     (*): passed on to deflatev and not used in deflate
%          Valid combinations are:
%     'size':    size of deflated matrix                
%                Only allowed when if argument "M" is not available.
%                Defaults to size(M) or [0,0] if M is not specified
%  (*)'emptyvalue': 
%                This value will be assumed for all elements that are not
%                specified.
%                Only allowed when if argument "M" is not available.
%                Defaults to double(0). 
%  (*)'mode'   : Storage mode: memory, compressed, disk
%                Defaults to compressed
%                compressed: equal to memory, but now the data blocks are
%                        compressed in memory. This expands the maximum 
%                        storage size considerably. If random access is
%                        required the CPU costs may be high.
%                memory: data is stored in memory in contiguous blocks of
%                        size "segmentSize". This expands the maximum
%                        storage size typically with a factor 3. compared
%                        to storage in contiguous blocks. The CPU costs are
%                        moderate.
%                disk:   TO BE IMPLEMENTED the data blocks are stored on
%                        disk. This is faster than compression, but also
%                        more vulnarable. Copies of the object point to the
%                        same file, unless a  "Deep" copy is made
%  (*)'segmentSize': smallest blocksize to break up vector
%                    Defaults to 100000
% 
% OUTPUT
%     m: deflated matrix corresponding to M
%   
% INTERNAL DATASTRUCTURE
%     m
%     +----size      : size of matrix stored
%     +----data      : deflatev object holding elements
% 
% EXAMPLE (1)
%     M=deflate('class','int16','size',[1000000,170]);
%     for k=1:size(M,2)
%         disp(k)
%         M(:,k)=int16(1:size(M,1));
%     end
% 
% EXAMPLE (2)
%     Verify methods "constructor", "struct", and "full"
%     a=rand(1000);is_eq(a,full(deflate(struct(deflate(a)))));

% REVISIONS
%     WIJZ ZIJPP 20100225: simplified code
%     WIJZ ZIJPP 20100225: derived segmentsize from matrix size by default

%Use this line to extract utility and all dependent functions
%extractapplic('deflate')

%-----------------------
%No arguments
%-----------------------
if nargin==0
    obj=deflate([]);
    return
end

%-----------------------
%1 or more arguments
%-----------------------
% Special case
% -1- implement deflate(obj), with obj of class deflate
if isa(varargin{1},'deflate')
    obj=varargin{1};
    if nargin>1
        error('When first argument is deflate object extra arguments are not allowed');
    end
    return
end
% Special case
% -2- implement deflate(S), with S a structure converted from a deflate object
if isstruct(varargin{1}) && nargin==1
    %make sure that all fields are present:
    if isfield(varargin{1},'size')&&isfield(varargin{1},'data')
        %all required fileds are there
        S=varargin{1};
        S.data=deflatev(varargin{1}.data);
        obj=class(S,'deflate');
        return
    end
end    

%-----------------------------------------------------------
%check if first argument sets data content or not
%-----------------------------------------------------------
if rem(nargin,2)==1
    %The first argument contains the data and implies size.
    S.size=size(varargin{1});
    %Provide default segment size argument derived from size. If segment
    %size is specified explicitly, it will overrule .
    segszprop=getSegSize(S.size);
    S.data  = deflatev(varargin{1},segszprop{:},varargin{2:end});
else
    %create, but do not populate deflated matrix. default values are
    %determined by "emptyvalue"
    S.size=readSizeArgument(varargin{:});
    %Provide default segment size argument derived from size. If segment
    %size is specified explicitly, it will overrule .
    segszprop=getSegSize(S.size);
    %data vector not specified, pass on "N" instead
    S.data  = deflatev(segszprop{:},varargin{:},'N',prod(S.size));
end
%-----------------------------------------------------------
%Initialize object
%-----------------------------------------------------------
obj=class(S,'deflate');
%__________________________________________________________________________
function sz=readSizeArgument(varargin)
ValidProps={'mode','segmentSize','emptyvalue','size'};
opt.size=[0 0];
opt=varargin2struct(opt,ValidProps,varargin{:});
sz=opt.size;
%__________________________________________________________________________
function segszprop=getSegSize(sz)
%Derive segment size from size. Set segment size equal to height of matrix,
%unless this leads to a very small or very high segment size.
if sz(1)>500 && sz(1)<10^6
    segszprop={'segmentSize',sz(1)};
else
    segszprop={};
end
