function [data]=subsref(obj,ind)
% subsref - subscripted reference in compressed vector
% 
%CALL
%  data=subsref(obj,ind)
%
%INPUT
%  obj: compressed vector
%
%  ind: struct array with fields
%         type: '()'
%         subs: cell array with subscript sets
%
% TEST
%     adata=rand(20000,200);
%     nw=now;
%     a=deflate(adata);
%     I1=1:3:300000;
%     a(I1)=I1;
%     adata(I1)=I1;
%     dprintf('T1=%f',86400*(now-nw));
%     bdata=full(a);
%     dprintf('T2=%f',86400*(now-nw));
%     is_eq(adata,bdata);

%check subscript
if ind(1).type(1)~='(' || length(ind)>1 || length(ind.subs)>1
    error('Subscripts in compressed vector should be of form: M(I1)');    
end

if strcmp(ind.subs,':')
    %efficient way to return all elements
    data=returnAll(obj);
    return
end
    
linIndices=ind.subs{1};
clear ind %from this point on structure is no longer needed

if max(linIndices)>numel(obj)    
    error('Index exceeds storage vector length.');
end

NOW=now; %plot waitbar if time exceeds 5 sec.

%initialize output
data=emptyData(obj,length(linIndices));

if isempty(linIndices)
    %returning now will prevent error in statement linIndices(end)
    return
end


if ~sortedindex(linIndices)
    %needed for ctranspose and others.
    [linIndices,indx]=sort(linIndices);
    indx=uint32(indx);
else
    indx=[];
end

if linIndices(end)>length(obj)
    error('Index exceeds matrix dimensions');
end

%indices are sorted now
[offsets,blks]=getOffsets(linIndices,obj.segmentSize);

%Local Memory claim (N=number of elements)
% blks       Nx1 uint16 array
% data       Nx1 CLASSTYPE array
% linIndices Nx1 uint32 array
% offsets    Nx1 uint32 array
  
[unblks,RANGE]=groupIndex(blks);
%Requires memory 2xN double ==> total 8+8+4+4+2+sizeof(CLASSTYPE)

N=1;
hwait=[];
TGAP=5/(1440*60);
for BLKINDX=1:length(unblks)
    blk=unblks(BLKINDX);
    hwait=waitbarc((now-NOW)>TGAP,N/length(unblks),hwait,'Retreive');
    N=N+1;
    
    %consecutive range, first and last have been found in groupIndex
    %(fast method)
    f=RANGE(BLKINDX,1):RANGE(BLKINDX,2);
    if ~isempty(obj.segment(blk).dat)
        data(f)=importRequested(obj,blk,offsets(f));
    end
end

if ~isempty(indx)
    %inverse sort is needed
    data(indx)=data;
end

%close waitbar if it was created
close(hwait);

function  data=returnAll(obj)
%return all data for an object
%no indices needed because contiguous block is retrieved

%initialize output
NOW=now;
data=emptyData(obj,length(obj));
TGAP=3/(1440*60);
hwait=[];

%now retrieve data from all segments and export data
for blk=1:length(obj.segment)
    hwait=waitbarc((now-NOW)>TGAP,blk/length(obj.segment),hwait,'Inflate');
    
    if ~isempty(obj.segment(blk).dat) %else: no action needed: leave zeros
        %retrieve data
        rng        = (blk-1)*obj.segmentSize +(obj.segment(blk).range(1):obj.segment(blk).range(2));
        data(rng)  = segmentData(obj.storemode,obj.segment(blk),obj.dataclass);        
    end
end
close(hwait);


