function data_out = compress(data_in,level)
% compress - compress array "data_in" to byte vector
%
% CALL:
%    data_out = compress(data_in,level)
%    level:        
%         <integer> (optional) compression level (1-9)
%                 1 -> best speed
%                 9 -> best compression
%
% INPUT
%     data_in(M,N): 
%         input array with numeric, charater or logical data
%         (cell arrays, structure arrays etc not allowed)
%
% OUTPUT
%    data_out(P,1): 
%         int8 vector that contains compressed variable
%
%
% NOTES (1)
%  When an array is compressed the size and class of the
%  original array can not be inferred from data_out. In order to
%  succesfully inflate the array again. uncompress requires the size
%  and class of the original array.
%
%       On march 30 compress, byte_mex, and uncompress are tested for
%       memory leaks. No leaks were found.
%
% NOTES (2)
%     -1-
%     There is a C and Java implementation for compression.
%     -2-
%     Un uncompression utility for Java is not currently available (20100929)
%     -3-
%     Compression using Java typically is twice as fast and therfore is the
%     preferred method
%     -4-
%     Uncompression typically is ten three times as fast as compression and is
%     currently carried out in C
%     -5-
%     Benchmarks using bench2 (with compress.m modified):
%     Class=uint8  compress-C: 0.0855 compress-J: 0.0420 uncompress-C: 0.0044
%     Class=uint16 compress-C: 0.1697 compress-J: 0.0854 uncompress-C: 0.0078
%     Class=uint32 compress-C: 0.3399 compress-J: 0.1768 uncompress-C: 0.0166
%     Class=single compress-C: 0.4784 compress-J: 0.2303 uncompress-C: 0.0736
%     Class=double compress-C: 0.8920 compress-J: 0.5581 uncompress-C: 0.1484
%     After modification compress.m was changed back
%     -6-
%     Compression is not fully supported for logicals: they are converted
%     to and from uint8
%     -7-
%     To do: fix uncompressJava and rerun benchmark
%
% EXAMPLE:
%  %create an example array with N elements
%  a=int16(100*rand(N,2000,100)); %or any ather type
%  aa = compress(a);
%  aaa =uncompress(aa,size(a),class(a));
%
% SEE ALSO:
%  uncompress
%  java.util.zip.Deflater/Inflater
%

if nargin==0
    bench2
    return

    %benchmark
    a=round(10000*rand(10^6+7,1))/10000;
    for k=1:9;
        bench(a,k);
    end

    a1=extensie(tempname,'mat');
    a2=extensie(tempname,'mat');
    tic;
    save(a1,'a');
    load(a1);
    dprintf('Save / Load     ==>   Time: %f',toc);

    tic
    save(a2,'-v6','a');
    load(a2);
    dprintf('Save -v6 / Load ==>   Time: %f',toc);
    delete(a1)
    delete(a2);
    return
end

if nargin==0
    N=16000;
    F1=1.2;
    F2=1.25;
    for k=1:10000
        dprintf('k=%d N=%d  Memory=%d',k,N,availableMemory);
        try
            a=rand(1000,N);
            %test for memory leak

            %             aa=deflate(a);
            %             aaa=full(aa);
            aa = compress(a);
            aaa =uncompress(aa,size(a),class(a));
            if ~is_eq(aaa,a)
                disp('Not equal');
            end
            N=round(F1*N);
        catch
            dprintf('Decrease N');
            N=round(N/F2);
        end
    end
end


%     %test for memory leak
%     if nargin==0
%         %check for memory leaks
%         for k=1:100
%             dprintf('K=%d Memory=%d',k,availableMemory);
%             sub=single(rand(4000,1000));
%             a=repmat(sub,10,1);
%
%             aa=deflate(a);
%             aaa=full(aa);
%     %         aa = compress(a);
%     %         aaa =uncompress(aa,size(a),class(a));
%             if ~is_eq(aaa,a)
%                 error('Not equal');
%             end
%     %         b=byte_mex(a);
%            clear a b aa aaa sub
%         end
%     end

if nargin==0
    error('At least 1 input argument required')
end

%NOTE ZIJPP 20160406
%modelit.jar and matlabserver.jar contain a java zip utility, but to use
%this tis one of these jars must be in static path. 
%Instead a mex file may be used. This is slightly slower, but easier to
%install and to include inapplications.

CUSAGE=true;
if CUSAGE
    % CODE BASED ON C-MEX FILE:
    %WIJZ ZIJPP 20101125: precompress logicals also in C mode
    if islogical(data_in)
        data_in=precompress(data_in);
    end
    data_out=compress_mex(data_in);
    return
end

%CODE BASED ON JAVA ZIP UTILS
if nargin <2
    level = 1;
end

%import zipUtil
import nl.modelit.utils.zip.*

%USE java routine because it is faster:
switch class(data_in)
    case 'logical'        
        data_in=precompress(data_in);
    case {'int8', 'uint8'}
        % no action
    otherwise
        %byte_mex converts vector of numbers to equivalent vector of bytes (uint8).
        %for example byte_mex(1)=[ 0    0    0    0    0    0  240   63]'
        data_in=byte_mex(data_in);
end
data_out=ZipUtil.compress(data_in,level);
%__________________________________________________________________________
function data=precompress(data)
% pack logical in uint8.
%numer of elements may not be 8-multiple
numbytes=numel(data)/8;
if rem(numbytes,1)
    %pad with false values
    numbytes=ceil(numbytes);
    data=data(:); %prevent error on matrix
    data((end+1):8*numbytes)=false;
end
data=logical2int(reshape(data,8,numbytes));
%note: the second byte differs between zipUtil and compress_mex but after
%      decompression no difference is observed

%__________________________________________________________________________
function bench(a,k)
sz1=numel(a)*sizeof(class(a));
tic;
b=compress(a,k);
aa=uncompress(b);
sz2=numel(b)*sizeof(class(b));
dprintf('Level: %d,  Time: %f  Factor %f',k,toc,sz2/sz1);

%__________________________________________________________________________
function bench2(a,k)
rand('seed',1);
data=sort(rand(800000,1));
types={'logical','uint8','uint16','uint32','single','double'};
% types={'uint8','uint16','uint32','single','double'};
disp(' ')
for k=1:length(types)
    type=types{k};
    switch type
        case 'logical'
            D=data>.5;
        case {'uint8','uint16','uint32'}
            D=feval(type,double(intmax(type))*data);
        otherwise
            D=feval(type,data);
    end
    tic;
    dComp1=compress(D,7);
    %compression levels do not seem to make much difference, only level 1
    %is worse then others 
    t1=toc;

    tic
    D1=uncompress(dComp1,size(D),type);
    t2=toc;
    assert(is_eq(D1,D),'error C');

    dprintf('Class=%10s compress: %.4f ==> CmpRate %.2f %% uncompress-C: %.4f',...
        type,t1,100*numel(dComp1)/(numel(D)*sizeof(type)),t2);
end


