function prop = getproperty(property, validprops, VERBOSE)
% getproperty - Return matching char string from cell array of keywords
%
% CALL:
%  prop = getproperty(property, validprops, VERBOSE)
%     
% INPUT:
%    property:   
%         String with property. This string contains the first letters of
%         the keyword searched for. The matching is Case-Insensitive. 
%    validprops: 
%         Cell array with valid property values
%     VERBOSE:
%         Defaults to true.
%         If true: display list with suggested options before throwing an
%         error 
%
% OUTPUT:
%    prop: 
%         String with property that matches validprops. An error will be
%         thrown in input argument "property" does not match any element of
%         "validprops"
%
% EXAMPLE:
%  getproperty('my',{'MySpecialProperty'}) returns 'MySpecialProperty'    
% 
%   Copyright 2006-2015 Modelit, www.modelit.nl
% 
% See also: rmproperty, varargin2struct


% REVISIONS
%     20040901: ZIJPP make algorithm more efficient by using strcmpi
%     20111111: ZIJPP, update help info. change capitalization 
%     20120504: ZIJPP, implement VERBOSE property

f=find(strncmpi(property,validprops,length(property)));
if isempty(f)
    
    %WIJZ ZIJPP 20120504: implement VERBOSE property
    if nargin<3
        %default behavior: VERBOSE=true
        VERBOSE=true;
    end
    if VERBOSE
        %Only plot info to console in verbose mode, otherwise throw error
        %message only
        
        %WIJZ ZIJPP 20120503: display more informative error messages
        disp(' ')
        disp('>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>');
        dprintf('Property "%s" was not recognized.',property);
        
        disp('Valid attributes (in alphabetical order) are:')
        
        [dummy,indx]=unique(lower(validprops));
        uvalidprops=validprops(indx);
        if length(validprops)>6
            %attempt column display
            MAXROWLEN=120;
            MAXW=size(char(uvalidprops),2);
            NUMCOL=floor(MAXROWLEN/(MAXW+2)); %number of columns
            NUMROW=ceil(length(uvalidprops)/NUMCOL);
            strs=cell(NUMROW,NUMCOL);
            strs(1:length(uvalidprops))=uvalidprops;
            disp(cell2str(strs,'   '));
        else
            disp(char(uvalidprops))
        end
        try
            disp(' ')
            stackdisp(dbstack);
            disp('Please refer to the codelines above.')
        catch
            %Modelit propriety code not found
        end
        
        disp('<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<');
        disp(' ')
    end
    error('Invalid attribute: %s',property);
end
if length(f)>1
	%WIJZ ZIJPP MAY 2006
	%Problem: if attrib name is extension of other. The shorter of the to
	%can no longer be identified.
	%Example: Autoresize & AutoResizeMode
	%Try to find exact match. If succesfull, use this one, if not, throw
	%error
    ff=find(strcmpi(property,validprops(f)));
    if length(ff)>1 || isempty(ff)
        disp('Matching attributes:');
        disp(char(validprops(f)));
        error('Ambiguous attribute name: %s',property);
    end
    f = f(ff);
end
prop=validprops{f};
