function equal=is_eq(varargin)
%is_eq - verify if argument pairs are equal
% 
% CALL
%     equal=is_eq(arg1,arg2,arg3,arg4,...)
% 
% INPUT
%     arg1,arg2: first argument pair
%     arg3,arg4: second argument pair
%     ...:       etc
% OUTPUT
%     equal: 1 if all argument pairs have corresponding size and are equal
% 
% NOTE
%     function is verbose if no output arguments are required
% 
% EXAMPLE (non verbose mode)
% 		>> a=is_eq(1,0)
% 		a =
%              0
% EXAMPLE (verbose mode)
% 		>> is_eq(1,0)
% 		Arg 1,2: Not equal, Max Abs Diff = 1.000000

if nargout
    VERBOSE=0;
else
    VERBOSE=1;
end
maxdif=0;
for k=1:2:length(varargin)
    M1=varargin{k};
    M2=varargin{k+1};
    thisdiff=isequal2(k,M1,M2,VERBOSE);
    if isnan(thisdiff)
        %difference is NaN skip others
        maxdif=NaN;
        break
    end
    maxdif=max(maxdif,thisdiff);
end
if nargout
    equal=(maxdif==0);
end

function mx=isequalCell(k,M1,M2,VERBOSE)
if ~iscell(M2)
    if VERBOSE
        dprintf('Arg %d,%d: Cell is not equal to Non-Cell',k,k+1);
    end
    mx=1;
    return
end
if ~all(size(M1)==size(M2))
    if VERBOSE
        dprintf('Arg %d,%d: Size of cell arrays does not match',k,k+1);
    end
    mx=1;
    return
end
mx=0;
for n=1:numel(M1)
    mx=max(mx,isequal2(k,M1{n},M2{n},VERBOSE));
end
%__________________________________________________________________________
function mx=isequalStruct(k,M1,M2,VERBOSE)
% Conmpare structure or structure arrays
% 
% INPUT
%     k: argument number
%     M1: first structure (array)
%     M2: second structure (array)
%     VERBOSE: display messages if true

%ZIJPP 20140320: verify that M2 is struct as well
if ~isstruct(M2)
    if VERBOSE
        dprintf('Arg %d,%d: Struct is not equal to Non-Struct',k,k+1);
    end
    mx=1;
    return
end

%ZIJPP 20140320: verify that sizes of M1 and M2 match
if~all(size(M1)==size(M2))
    if VERBOSE
        dprintf('Arg %d,%d: Size of struct arrays does not match',k,k+1);
    end
    mx=1;
    return
end

if numel(M1)>1
   
    mx=0;
    for n=1:numel(M1)
        %when comparing struct araays, only provide info once
        mx=max(mx,isequalStruct(k,M1(n),M2(n),false)); %WIJZ ZIJPP DEC 08
        if mx~=0
            break
        end
    end
    %     Revision, Zijpp, Dec 2008: comparison of structure arrays did not
    %     work properly. This problem has now been corrected.
    if VERBOSE
        if mx~=0
            %repeat last comparison in verbose mode
            isequalStruct(k,M1(n),M2(n),true);
        else
            dprintf('All %d elements of structure array equal',numel(M1));
        end
    end
    return %ADDED ZIJPP DEC 08
end


sdif1=setdiff(fieldnames(M1),fieldnames(M2));
if ~isempty(sdif1)
    if VERBOSE
        for k=1:length(sdif1)
            dprintf('Field %s in Arg %d but not in Arg %d',sdif1{k},k,k+1);
        end        
    end
end
sdif2=setdiff(fieldnames(M2),fieldnames(M1));
if ~isempty(sdif2)
    if VERBOSE
        for k=1:length(sdif2)
            dprintf(' Field %s in Arg %d but not in Arg %d',sdif2{k},k+1,k);
        end        
    end
end
if ~isempty(sdif1)||~isempty(sdif2)
     mx=1;
    return
end

flds=fieldnames(M1);
mx=0;

for kk=1:length(flds)
    fld=flds{kk};
    if VERBOSE
        dprintf('Compare field "%s"',fld);
    end
    thismx=isequal2(k,M1.(fld),M2.(fld),VERBOSE);
    if VERBOSE&&thismx
        dprintf('Structure field "%s" not equal',fld);
    end
    mx=max(mx,thismx);
end
%__________________________________________________________________________
function mx=isequal2(k,M1,M2,VERBOSE)
% INPUT
%     k: argument number
%     M1: first of pair
%     M2: second of pair
 
   
if iscell(M1)
    %NEW: compare cell arrays ZIJPP OCT 2005
    mx=isequalCell(k,M1,M2,VERBOSE);
    return
end

if isempty(M1) && isempty(M2)
    mx=0;
    if VERBOSE
        dprintf('Arg %d,%d: Both empty',k,k+1);
    end
    return
end
if isstruct(M1)
    %NEW: compare struct ZIJPP APR 2006
    mx=isequalStruct(k,M1,M2,VERBOSE);
    return
end
%ZIJPP 20140306: insert object comparison
if isobject(M1)&&isobject(M2)
    mx=isequal2(k,struct(M1),struct(M2),VERBOSE);
    return
end
if all(size(M1)==size(M2))
    %try and find difference
    %WIJZ ZIJPP 20100802: deeal with sparse matrices
    if issparse(M1)
        if ~issparse(M2)
            if VERBOSE
                dprintf('Arg %d sparse,Arg %d not sparse',k,k+1);
                mx=1;
                return
            end
        end
        f=find(M1~=M2);
        if ~isempty(f)
            f=f(1);
        end
    else
        f=find(M1~=M2,1,'first');
    end
            
    if isempty(f)
        %all elements are equal and No NaN values are present
        if VERBOSE
            dprintf('Arg %d,%d: Equal',k,k+1);
        end
        mx=0;
        return
    end
    
    %values are not equal or NaN values occur    
        
    %WIJZ ZIJPP: get the max difference in a memory friendly way
    STEP=10^6;
    mx=-inf;
    mxindx=0;
    START=1;
    N=numel(M1);
    while 1
        rng=START:min(START+STEP-1,N);
        
        %WIJZ ZIJPP DEC 2005
        %check if NaN values correspond
        if any(isnan(M1(rng))~=isnan(M2(rng)))
            if VERBOSE
                dprintf('Arg %d,%d: Not equal, NaN values do not correspond',k,k+1);
            end
            mx=NaN;
            return
        end
        
        %NaN values correspond
        thisrng=rng(~isnan(M1(rng)));
        [thismax,thisindx]=max(abs(double(M1(thisrng))-double(M2(thisrng))));
        if thismax>mx
            mx=thismax;
            mxindx=thisrng(thisindx);
        end
        if rng(end)==N
            break;
        end
        START=START+STEP;
    end
       
    if VERBOSE        
        if mx>0
            dprintf('Arg %d,%d: Not equal, Max Abs Diff = %f (index=%d)',k,k+1,mx,mxindx);
        else
            dprintf('Arg %d,%d: Equal or both NaN',k,k+1);
        end
    end

else
    if VERBOSE
        dprintf('Arg %d,%d: Size does not match (Arg%d: %d elms, Arg%d: %d elms)',...
            k,k+1,...
            k,prod(size(M1)),...
            k+1,prod(size(M2)));
    end
    mx=1;
end