function [linindex,dims]=sub2indVector2(sz,range,varargin)
% SUB2INDVECTOR2 equivalent to SUB2INDVECTOR, but returns specified subset
% of indices
% 
% CALL
%     [linindex,dims]=sub2indVector2(SIZ,RANGE,I1,I2,...,In)
%     
% INPUT
%     SIZ: size of matrix to index in
%     RANGE: the indices of subscripts required. RANGE indices in a matrix
%            implied by I1,I2,...,In
%     I1,I2,...,In: subscript array to be used
% 
% OUTPUT
%     linindex: [UINT32] linear index corresponding to I1,I2,...,In
%     dims: size of required dimensions
%           length(linindex)=prod(dims);
%           After:
%               [linindex,dims]=sub2indVector(size(M),I1,I2,...,In)
%           The following applies:        
%               M(I1,I2,...) 
%           is equivalent to: 
%               reshape(M(linindex),dims)
%         
% SEE ALSO
%     sub2indVector
% 
% NOTES
%     ** Equivalent command:
%       sub2indVector2(sz,1:M,I1,I2,...,In)  
%       (M=number of indices implied by I1,.. In)
%     is equivalent to:
%       sub2indVector(sz,I1,I2,...,In) 
%     ** sub2indVector2 should only be used if there is not enough memory
%     available to apply sub2indVector  !!
%     ** sub2indVector2 takes about 5 times the CPU time of sub2indVector
%     

%TEST
% [linindex,dims]=sub2indVector2([8000 35136],uint32(195000000:195250752),1:5557,1:35136);

% Nanne van der Zijpp
% Modelit

% if nargin==0
%     %testmode
%     linindex=sub2indVector([4 4],[1 3],[2 4])
%     return;
% end

% if nargin==0
%     %test/benchmark
%     tic
%     L1=sub2indVector([1233,345,66],1:20,3:333,2:55);
%     dprintf('Method 1: %f',toc);
%     tic
%     L2=sub2indVector2([1233,345,66],1:20*331*54,1:20,3:333,2:55);
%     dprintf('Method 2: %f',toc);
%     is_eq(double(L1(:)),L2(:));
%     return
% end

for k=1:length(varargin)
    %return vertical vector
    varargin{k}=varargin{k}(:);
end

STEP=5*10^5;
L=uint32(length(range));
first=uint32(1);
linindex=zeros(size(range),'uint32');

%do work in small portions so that it fits in memory
while first<=length(range)
    last=min(first+STEP,L);
    rng=first:last;    
    [linindex(rng),dims]=sub2indVector2_compute(sz,range(rng),varargin{:});
    first=last+1;
end
%----------------------------------
function [linindex,dims]=sub2indVector2_compute(sz,range,varargin)

range=double(range); %do aritmetic in singles
%WARNING: a single has unsufficient digits to hold large index
    
%If number of subscripts is larger then number of dimensions append "1" for
%missing dimensions
sz(length(sz)+1:length(varargin))=1;

%compute the total number of required indices
for k=1:length(varargin) 
    dims(k)=length(varargin{k});
end
Ntot=prod(dims);

[subIndex{1:length(dims)}]=ind2sub(dims,range);
%subIndex: index to matrix of indices

for k=1:length(varargin)
    subIndex2{k}=varargin{k}(subIndex{k});
    %subIndex2: index in matrix of size sz
end
linindex=uint32(sub2ind(sz,subIndex2{:}));


